// Unreal Fighting Framework by Kevin Poretti

#include "FFInputBufferComponent.h"

UFFInputBufferComponent::UFFInputBufferComponent()
{
	PrimaryComponentTick.bCanEverTick = false;
}

void UFFInputBufferComponent::AddInput(const FFFInputState& InputState)
{
	InputBuffer.ForcePush(InputState);
}

bool UFFInputBufferComponent::CheckInputSequence(const FFFInputSequence& InputSequence)
{
    int CondIdx = InputSequence.Sequence.Num() - 1;
    int FramesSinceValidInput = 0;
    for(int InpIdx = 0; InpIdx < InputBuffer.Num() - 1; InpIdx++)
    {
        int32 RequiredButtons = InputSequence.Sequence[CondIdx].RequiredButtons;
        EFFButtonState RequiredButtonState = InputSequence.Sequence[CondIdx].RequiredButtonState;
        int32 PrevInput = InputBuffer[InpIdx + 1].Buttons;
        int32 CurrInput = InputBuffer[InpIdx].Buttons;
        int32 PrevDisable = InputBuffer[InpIdx + 1].DisabledButtons;
        int32 CurrDisable = InputBuffer[InpIdx].DisabledButtons;
        switch (RequiredButtonState)
        {
            case EFFButtonState::BTNS_Pressed:
                if(!(PrevInput & RequiredButtons | PrevDisable) && 
                   CurrInput & RequiredButtons & ~CurrDisable)
                {
                    CondIdx--;
                    FramesSinceValidInput = 0;
                }
                break;
            case EFFButtonState::BTNS_Released:
                if(PrevInput & RequiredButtons & ~PrevDisable &&
                   !(CurrInput & RequiredButtons | CurrDisable))
                {
                    CondIdx--;
                    FramesSinceValidInput = 0;
                }
                break;
            // TODO: implement button held condition
            /*
            case EFFButtonState::BTNS_Held:
                break;
            */
        }

        // All conditions were met
        if(CondIdx == -1)
        {
            // disable inputs that triggered the sequence
            InputBuffer[InpIdx].DisabledButtons |= InputBuffer[InpIdx].Buttons;
            InputBuffer[InpIdx + 1].DisabledButtons |= InputBuffer[InpIdx + 1].Buttons;
            return true;
        }

        FramesSinceValidInput++;
        if(FramesSinceValidInput > InputSequence.Sequence[CondIdx].Lenience)
        {
            return false;
        }
    }

	return false;
}
